#!/usr/bin/perl

# Helper library for dnswl.org client scripts. Licensed
# under the GNU GPL. 
#
# $Id: dnswl-lib.pl,v 1.3 2006/11/26 20:37:25 mleisi Exp $

use strict;
use warnings;

# Configuration files can be passed as the single paramter
# on the command line or are best-guessed according to 
# @confdirs below. It is NOT guaranteed that the file will
# exist when the caller wants it.
sub findConfig {
	my @confdirs = qw|/etc/dnswl.conf /opt/dnswl/dnswl.conf ./dnswl.conf ~/dnswl.conf ~/dnswl/dnswl.conf|;
	
	if (@ARGV > 0) {
		return $ARGV[0]; 
		
	} else {
		foreach my $f (@confdirs) {
			if (-f $f) {
				return $f;
			}
		}	
	}
}

# Encapsulate rsync calls. Return values are the same as
# those from the rsync binary, see rsync(1)
sub rsync {
	my $remotefile = shift;
	my $localfile = shift;
	my $cmd = sprintf('%s --address=%s --times %s/%s %s',
		$dnswl::rsync, $dnswl::bind_address, $dnswl::rsync_url,
		$remotefile, $localfile); 
	if ($dnswl::debug == 1) {
		print "$cmd\n";
	}
	`$cmd`;
	return $?;
	
}

# Encapsulate GnuPG calls. Return values are the same as 
# those from the gpg binary, see gpg(1)
sub pgp {
	my $file = shift;
	my $sig = shift;

	my $d = '';
	if ($dnswl::debug == 0) {
		$d = '2>&1 > /dev/null';
	}
	
	my $cmd = sprintf('%s --quiet --verify %s %s %s', $dnswl::pgp, $sig, $file, $d);
	if ($dnswl::debug == 1) {
		print "$cmd\n";
	}
	`$cmd`;
	return $?;
	
}

# Encapsulate wget calls. Return values are the same as
# those from the wget binary, see wget(1)
sub wget {
	my $url = shift;
	my $file = shift;

	my $cmd = sprintf('%s --quiet --output-document %s %s', $file, $url);
	if ($dnswl::debug == 1) {
		print "$cmd\n";
	}
	`$cmd`;
	return $?;
}

# Encapsulate PGP verification; also attempts to download
# the key if it is not present in the calling user's keyring.
# If PGP verification is not enabled, return value will always
# be 1 (success). Return values greater than 1 indicate some 
# kind of failure: 2 = PGP key not available, 3 = Signature
# file not available, 4 = Signature verification failed. 
sub verify {
	my $file = shift;
	my $sig = shift;

	if ($dnswl::pgp_verify == 1) {
		my $key_available = fetch_key();
		if ($key_available == 1) {
			my $t = sprintf('%s/%s', $dnswl::tmpdir, $sig);
			if (!(rsync($sig, $t) == 0)) {
				return 3;	# Signature file not available
			}
			my $pgp_result = pgp($file, $t);
			
			if ($pgp_result == 0) {
				unlink($t);
				return 1;	# PGP verification succeeded
			} else {
				return 4;	# PGP verification tried, but failed
			}
		} else {
			return 2;		# PGP key not available
		}
	} else {
		return 1;			# PGP verification skipped
	}
}

# Encapsulate public key download and insertion into 
# keyring of the calling user. Will first check if the
# key is available. If the key is available (either 
# already present or successfully downloaded), the return
# value is 1; on download failure the return value is 2.
#
# TODO: code not heavily tested, converted from a similar
# shell script.
sub fetch_key() {

	my $cmd = sprintf('%s --list-keys --fingerprint | grep "%s" 2>&1 > /dev/null',
		$dnswl::pgp, $dnswl::pgp_fingerprint);
	print "$cmd\n" if ($dnswl::debug == 1);
	
	if ($? == 1) {
		if ($dnswl::debug == 1) {
			print "Importing PGP key from $dnswl::pgp_keyurl\n";
		}
		my $t = sprint('%s/pubkey.asc', $dnswl::tmpdir);
		if (!(wget($dnswl::pgp_keyurl, $t) == 0)) {
			print "Could not fetch key from $dnswl::pgp_keyurl\n";
			return 2;
		}
	}
	
	return 1;
#	$PGP --list-keys --fingerprint | grep "$FINGERPRINT" 2>&1 > /dev/null
#	if [ "1" = "$?" ]; then
#		echo "Importing PGP key from $KEYURL"
#		$WGET --quiet --output-document $OUTDIR/pubkey.asc $KEYURL
#		if [ "0" = "$?" ]; then
#			$PGP --quiet --import $OUTDIR/pubkey.asc
#			rm -f $OUTDIR/pubkey.asc
#		else
#			echo "Error: Could not get PGP key from $KEYURL"
#		fi
#	
#		$PGP --list-keys --fingerprint | grep "$FINGERPRINT" 2>&1 > /dev/null
#		if [ "1" = "$?" ]; then
#			echo "Error: signature verification not available"
#			return 1
#		else
#			echo "*********************************************************************"
#			echo "* Successfully imported PGP key (fingerprint=$FINGERPRINT)"
#			echo "* You will need to certify this key with a trusted signature"
#			echo "* before PGP verification will succeed:"
#			echo "* | gpg --sign-key $KEYID"
#			echo "*********************************************************************"
#			return 0
#		fi
#		echo ""
#    else
#		return 0
#	fi
}

# Make require happy:
1;
